
#ifndef ARAIGNEE_H
#define ARAIGNEE_H

//#include "Molecule.h"
#include "Atom.h"
#include "pi2a14_patte.h"


class Araignee : public API::Molecule
{
private:
	
	/**
	 * les pattes de l'araigne
	 */
	Patte * pattes;
	
	/**
	 * les atomes qui composent l'araigne (il sont juste la pour faciliter la gestion etant donne qu'ils sont
	 * aussi presents dans la class parente Molecule
	 */
	API::Atom * atoms;
	
	/**
	 * le type du mouvement en cours exemple : 1 = marche, 2 = rammaser objet, 3 = straf
	 */
	int moveType;
	
	/**
	 * l'index de l'etape courante du mouvement courant, permet de discretiser le mouvement
	 */
	int etape;
protected:
public:
	/**
	 * constructeur par defaut, construit une araigne avec 12 atoms
	 */
	Araignee();
	
	/**
	 * destructeur par defaut
	 */
	~Araignee();
	
	/**
	 * methode qui initialise l'araigne en descendant son centre de gravit
	 */
	void initialise();
	
	/**
	 * methode qui lance le mouvement de marche vers l'avant
	 */
	void avance();
	
	/**
	 * methode qui lance le mouvement pour rammaser un objet au sol : l'araigne se baisse et quand elle est
	 * suffisament basse ou que les patte en bas du corp de l'araigne touchent le sol, elle attrape l'objet.
	 */
	void ramasser();
	
	/**
	 * methode qui lance le mouvement d'attrapage d'objet. ce mouvement permet  l'araigne d'aprehender des objets
	 * en refermant les pattes du bas des atoms qui forment le corps vers le centre.
	 */
	void attraper();
	
	/**
	 * methode qui verifie les synchronisation de la molecule. si une condition de synchronisation est valide
	 * alors la methode passe  l'etape suivante.
	 */
	void checkSynchro();
	
	/**
	 * methode qui lance le mouvement de straf (marche en crabe) vers la gauche.
	 * pour des raisons d'equilibre ce mouvement ne marche pas bien.
	 */
	void strafGauche();
};


#endif
