//********************************************
//Test de l'araigne
//Par Michel
//********************************************

#include <string>

#include <drawstuff/drawstuff.h>
#include <ode/ode.h>

#include <time.h>
#include <sys/time.h>

#include "ConfigReader.h"

#include "mmgr.h"

#include "CollisionGroup.h"
#include "CollisionManager.h"
#include "Engine.h"
#include "Entity.h"
#include "Event.h"
#include "SimWorld.h"
#include "Math.h"
#include "RigidBody.h"
#include "HingeJoint.h"
#include "AMotorJoint.h"
#include "SimServoMotor.h"
#include "Simulator.h"
#include "Timer.h"
#include "Atom.h"
#include "Graph.h"


#include "pi2a14/pi2a14_araignee.h"

#define ATOM_UNIT (DVAR(KERNELSIDE)+2*DVAR(LEGSIDE))


/*******************************/

CR_DECLARE (TARGET_FPS);
CR_DECLARE (GROUP_POSITION);
CR_DECLARE (ATOM_POSITION);
CR_DECLARE (GROUP_ATOM_COUNT);
CR_DECLARE (GROUP_ATOM_PER_ROW);
CR_DECLARE (GROUP_ATOM_PER_LINE);
CR_DECLARE (KERNELSIDE);
CR_DECLARE (SPHERESIDE);
CR_DECLARE (LEGRADIUS);
CR_DECLARE (LEGSIDE);


Engine*         engine         = NULL;
CollisionGroup* collisionGroup = NULL;


Araignee* araignee;


dJointID fixation;

int mode;

void linkAtom(API::Atom * atom1, API::Atom * atom2)
  {
  bool link = false;

  for (int i = 0;i < 6; i++)
    {
    for (int j = 0; j < 6; j++)
      {
      if (atom1->getLeg(i)->util->isConnectable(atom2->getLeg(j)))
        {
        atom1->getLeg(i)->setLink(atom2->getLeg(j));
        printf("patte %d et patte %d connect\n",i,j);
        link = true;
        }
      if (link)
	{
	break;
	}
      }
      if (link)
	{
	break;
	}
      } 
  }

void sequence()
  {

  }

void step(int pause)
  {
  engine->beginFrame();
  
  araignee->checkSynchro();
  
  engine->frame();
  engine->endFrame();
  }

void stop()
  {
  delete araignee;

  delete collisionGroup;
  printf("Dsallocation effectue !");
  engine->shutdown();
  }


void command(int cmd)
  {
  switch(cmd)
    {
    case 'p' : engine->setIsPaused(! engine->getIsPaused()); break;

    
		case 'l':
			araignee->initialise();
			break;
		case 'n':
			araignee->avance();
			break;
		case 'a':
			araignee->ramasser();
			break;
		case 'm':
			araignee->attraper();
			break;
		case 's':
			araignee->strafGauche();
			break;



    default : printf("Touche non reconnue : %c (%i)\n", cmd, cmd);
    }
  }

int main(int argc, char** argv)
  {
  CR_SETUP(Main, TARGET_FPS         , CR_DOUBLE);
  CR_SETUP(Main, GROUP_POSITION     , CR_DOUBLE);
  CR_SETUP(Main, ATOM_POSITION      , CR_DOUBLE);
  CR_SETUP(Main, GROUP_ATOM_COUNT   , CR_DOUBLE);
  CR_SETUP(Main, GROUP_ATOM_PER_ROW , CR_DOUBLE);
  CR_SETUP(Main, GROUP_ATOM_PER_LINE, CR_DOUBLE);

  CR_SETUP(Atom, KERNELSIDE, CR_DOUBLE);
  CR_SETUP(Atom, SPHERESIDE, CR_DOUBLE);
  CR_SETUP(Atom, LEGRADIUS , CR_DOUBLE);
  CR_SETUP(Atom, LEGSIDE   , CR_DOUBLE);

  char *odehome_env = getenv("ODEHOME");
  std::string odehome_string;

  #ifdef USE_OPENGL

  dsFunctions fn;

  fn.version = DS_VERSION;
  fn.start   = 0;
  fn.step    = &step;
  fn.command = &command;
  fn.stop    = &stop;

  if(odehome_env == NULL)
    odehome_string = "../../ode/drawstuff/textures";
  else
    {
    odehome_string = odehome_env;
    odehome_string += "/drawstuff/textures";
    }

  fn.path_to_textures = (char*)odehome_string.c_str();

  #endif

  engine = Engine::getInstance();
  engine->initialize();
  engine->setTargetFps(DVAR(TARGET_FPS));

  collisionGroup = new CollisionGroup();

  SimMolecule::setCollisionGroup(collisionGroup);

  CollisionManager::getInstance()->addCollisionGroup(collisionGroup);

  Node *root = engine->getRoot();

  collisionGroup->addCollidee(root);

  SimWorld *world = new SimWorld();

  root->addChild(world);

  //***************

  araignee = new Araignee();
  //***************

  #ifdef USE_OPENGL

  printf("J'utilise OpenGL !\n");

  dsSimulationLoop(argc, argv, 640, 480, &fn);

  #else

  printf("Je n'utilise pas OpenGL !\n");

  bool quit  = false;
  bool pause = false;

  printf("Starting main loop...\n");

  engine->setIsPaused(true);

  while(! quit)
    step(pause);

  printf("Stopping !!!");

  stop();

  #endif

  return(0);
  }
