/*************************************************************************************************************************
 *
 *
 *                                 Walker, le quadripde
 *
 *                                   Frdric Castellier
 *
 *
 * date 	: 09/12/2003
 * revision 	: 10/12/2003
 ************************************************************************************************************************/

#include <string>

#include <drawstuff/drawstuff.h>
#include <ode/ode.h>

#include <time.h>
#include <sys/time.h>
#include <stdlib.h>

#include "ConfigReader.h"

#include "mmgr.h"

#include "CollisionGroup.h"
#include "CollisionManager.h"
#include "Engine.h"
#include "Entity.h"
#include "Event.h"
#include "SimWorld.h"
#include "Math.h"
#include "RigidBody.h"
#include "HingeJoint.h"
#include "AMotorJoint.h"
#include "SimServoMotor.h"
#include "Simulator.h"
#include "Timer.h"
#include "Atom.h"
#include "Graph.h"

#include "pi2a14/pi2a14_walker.h"   

/*******************************/


/**
 * Qu'est ce que ca veut dire ????
 *
 */
CR_DECLARE (TARGET_FPS);
CR_DECLARE (GROUP_POSITION);
CR_DECLARE (ATOM_POSITION);
CR_DECLARE (GROUP_ATOM_COUNT);
CR_DECLARE (GROUP_ATOM_PER_ROW);
CR_DECLARE (GROUP_ATOM_PER_LINE);

CR_DECLARE (KERNELSIDE);
CR_DECLARE (SPHERESIDE);
CR_DECLARE (LEGRADIUS);
CR_DECLARE (LEGSIDE);

/**
 * l'instance du simulateur je suppose
 */

Engine * engine = 0;

/**
 * pour la gestion des collisions
 */
CollisionGroup* collisionGroup = 0;

/**
 * le nombre d'atomes de la simulation
 */
int atomNumber = 3;

/**
 * notre atome magique
 * j'ai mis un pointeur pour pouvoir reutiliser ce fichier pour le test unitaire a 2 atomes
 */
API::Atom * atome;

/**
 * permet d'accrocher l'atome a l'environnement
 */
dJointID fixation;

/**
 * la patte active
 */
int patteActive = 0;

/**
 *
 */
int atomeActif = 0;

/**
 * le mode d'affectation des ordres des pattes (KEEP ou REPLACE)
 */
int mode;
Walker *walkerMol;

/**
 * fonction appele a chaque image du simulateur
 * @param int pause : {vrai/1} si le simulateur doit etre arret {faux/0} si le simulateur tourne normalement
 */
void step(int pause)
{
	// debut de l'affichage
	// initialise openGL et tout le tintouin pour l'affichage
	engine->beginFrame();

	walkerMol->checkSynchro();
	//printf("x:%f",atome[0].util->getCartesianPosition().getX());
	// ODE calcule, OpenGL affiche et l'utilisateur glande si ca rame :-)
	engine->frame();

	// on termine l'affichage
	engine->endFrame();
}

/**

 * fonction appele quand on veut quitter
 *
 */
void stop ()
{
  for (int i = 0 ; i < atomNumber ; i++)
  {
	// on supprime les atome de la memoire
	delete atome;
	atome = 0;
  }
  // et on suprime le groupe de collision 
  delete collisionGroup;
  printf("Desallocation faite !");
  engine->shutdown ();
}
 
/** 
 * methode appel lors de l'appuye sur une touche
 * @param int cmd : la valeur ASCII de la touche appuy
 */
void command (int cmd) 
{ 
	switch (cmd) 
	{ 
		case 'p':// on lance ou arrete le rendu
			engine->setIsPaused (!engine->getIsPaused ());
			break;
		case 'i': walkerMol->initStruct();
			break;
		case 'r': walkerMol->turnRight();
			break;
		case 'l': walkerMol->turnLeft();
			break;
		case 'a': walkerMol->triang();
			break;
		case 'w': walkerMol->walk();
			break;
		case 's': walkerMol->strafRight();
			break;
		case 'q': walkerMol->strafLeft();
			break;
	}
}

int main (int argc, char** argv)
{
  /**
   * toujours le mystere a propos de ces "fonctions"
   */
  CR_SETUP (Main, TARGET_FPS, CR_DOUBLE);
  CR_SETUP (Main, GROUP_POSITION, CR_DOUBLE);
  CR_SETUP (Main, ATOM_POSITION, CR_DOUBLE);
  CR_SETUP (Main, GROUP_ATOM_COUNT, CR_INT);
  CR_SETUP (Main, GROUP_ATOM_PER_ROW, CR_INT);
  CR_SETUP (Main, GROUP_ATOM_PER_LINE, CR_INT);

  CR_SETUP (Atom, KERNELSIDE, CR_DOUBLE);
  CR_SETUP (Atom, SPHERESIDE, CR_DOUBLE);
  CR_SETUP (Atom, LEGRADIUS, CR_DOUBLE);
  CR_SETUP (Atom, LEGSIDE, CR_DOUBLE);

  /**
   * donne le "home" d'ode
   */
  char* odehome_env = getenv("ODEHOME");
  std::string odehome_string;

 

  /**
   * code qui ne sera compil que si USE_OPENGL exist
   */
#ifdef USE_OPENGL 
	/**
	 * vive le copi coll
	 * je sait toujours pas c'est qui ce fn...
	 * chazam ca y est je sait ce que c'est cf:drawtuff/drawstuff.h
	 * en fait c'est une structure qui contient un pointeur vers une fonction de depart
	 * un pointeur vers une fonction d'affichage
	 * un pointeur vers une fonction de commande
	 * un pointeur vers une fonction appel a la fermeture du programme
	 * et l'addresse des textures pour le simulateur...
	 */
	dsFunctions fn;
	fn.version          = DS_VERSION;
	fn.start            = 0;
	// un pointeur vers la fonction d'affichage
	fn.step             = &step;
	// idem vers la fonction de commande
	fn.command          = &command;
	// et de stop
	fn.stop             = &stop;
	// mais pourquoi il n'ont pas utilis GLUT ?????
	// la est la question (ceci etait un delire perso :-) )

	// on lui precise ou sont les fonction d'affichage d'ODE
	if (odehome_env == NULL)
	{
		odehome_string = "../../ode/drawstuff/textures";
	}
	else
	{
		odehome_string = odehome_env;
		odehome_string += "/drawstuff/textures";
	}

	// et on lui donne le chemin des textures
	fn.path_to_textures = (char*)odehome_string.c_str ();
#endif

	// on recupre l'instance unique d'Engine (pattern singleton)
	engine = Engine::getInstance();

	// on l'initialise
	engine->initialize();

	// et la on lui dit de faire un truc (a propos du nombre d'images par secondes
	engine->setTargetFps(DVAR (TARGET_FPS));

 // on cre un nouveau groupe de collisions
	// j'espere que ca a ete declar dans un .h quelque part car la je voit pas...
	collisionGroup = new CollisionGroup ();

	// et on set le groupe de collision sur les molecules... (ca veut rien dire sortie du contexte)
	SimMolecule::setCollisionGroup(collisionGroup);

	// on recupere l'instance unique de CollisionManager et on lui ajoute le groupe de collision
	CollisionManager::getInstance ()->addCollisionGroup (collisionGroup);

	// on recupere la racine de l'Engine
	Node* root    = engine->getRoot  ();

	// et on ajoute la racine au groupe de collision
	collisionGroup->addCollidee (root);


	// on cree le monde (et le monde fut ... )
	SimWorld* world = new SimWorld ();

	dWorldSetGravity (Simulator::getInstance()->getWorld(), 0, 0, -9.81);
	// et on ajoute le niveau comme enfant de la racine... faut vraiment que je regarde comment marche les
	// collisions d'ODE
	root->addChild (world);

	//Cration d'une molcule de type Fred1 
	walkerMol = new Walker();
	

#ifdef USE_OPENGL
	// on initialise la fenetre (finalement ils utilisent peut etre GLUT :-) ) en 640*480 et avec l'adresse de
	// ce p****n de fn
	// bon maintenant que je sait ce que c'est c'est bon... (cf: debut de la methode main)
	dsSimulationLoop (argc, argv, 600, 480, &fn);
#else


	// on initialise les variables qui seront utilise par la boucle de restitution
	bool quit = false;
	bool pause = false;

	// on affiche un message a la con
	printf("Starting main loop...\n");

	// on dit a l'enine qu'il ne doit pas etre stopp au demarrage (d'ailleur ce serait con car vu qu'il est
	// stop on ne peut pas appeler la fonction commande qui pourrait le debloquer (cf: buge possible du simulateur
	// de base) )
	engine->setIsPaused (true);

	// boucle de restitution de moteur
	while (!quit) {
		step(pause);
	}

	// hop encore un affichage de message a la con :-)
	printf("Stopping!!\n");

	// et on stop le tout...
	stop();
#endif

	// return 0 => cf: c ANSI
	return 0;
}



