//********************************************
//Test unitaire avec un seul atome
//Par Titi
//********************************************

#include <string>

#include <drawstuff/drawstuff.h>
#include <ode/ode.h>

#include <time.h>
#include <sys/time.h>

#include "ConfigReader.h"

#include "mmgr.h"

#include "CollisionGroup.h"
#include "CollisionManager.h"
#include "Engine.h"
#include "Entity.h"
#include "Event.h"
#include "SimWorld.h"
#include "Math.h"
#include "RigidBody.h"
#include "HingeJoint.h"
#include "AMotorJoint.h"
#include "SimServoMotor.h"
#include "Simulator.h"
#include "Timer.h"
#include "Atom.h"
#include "Graph.h"

/*******************************/

CR_DECLARE (TARGET_FPS);
CR_DECLARE (GROUP_POSITION);
CR_DECLARE (ATOM_POSITION);
CR_DECLARE (GROUP_ATOM_COUNT);
CR_DECLARE (GROUP_ATOM_PER_ROW);
CR_DECLARE (GROUP_ATOM_PER_LINE);
CR_DECLARE (KERNELSIDE);
CR_DECLARE (SPHERESIDE);
CR_DECLARE (LEGRADIUS);
CR_DECLARE (LEGSIDE);


Engine*         engine         = NULL;
CollisionGroup* collisionGroup = NULL;

int atomNumber = 1;

API::Atom *atome;

dJointID fixation;

int patteActive = 0;
int atomeActif   = 0;

int mode;

void afficheEtat()
  {
  printf("-----------------\n");
  printf("Atome actif  : %i\n", atomeActif+1);
  printf("Patte active : %i\n", patteActive+1);
  }

void sequence()
  {

  }

void step(int pause)
  {
  engine->beginFrame();
  engine->frame();
  engine->endFrame();
  }

void stop()
  {
  for(int i = 0; i < atomNumber; i++)
    {
    delete atome;
    atome=0;
    }

  delete collisionGroup;
  printf("Dsallocation effectue !");
  engine->shutdown();
  }


void command(int cmd)
  {
  switch(cmd)
    {
    case 'p' : engine->setIsPaused(! engine->getIsPaused()); break;

    case '1' : patteActive = 0; afficheEtat(); break;
    case '2' : patteActive = 1; afficheEtat(); break;
    case '3' : patteActive = 2; afficheEtat(); break;
    case '4' : patteActive = 3; afficheEtat(); break;
    case '5' : patteActive = 4; afficheEtat(); break;
    case '6' : patteActive = 5; afficheEtat(); break;

    case '+' : atomeActif++;
               if(atomeActif>atomNumber-1)
	         atomeActif=0;
	       afficheEtat();
	       break;

    case '-' : atomeActif--;
               if(atomeActif+1<0)
	         atomeActif=atomNumber;
               afficheEtat();
	       break;

    case 'i' : for(int i=0; i<atomNumber; i++) //Palace les pattes au centre
                  for(int j=0; j<6; j++)
                    {
		    atome[i].getLeg(j)->getServoMotor(0)->setTargetPosition(125, REPLACE);
		    atome[i].getLeg(j)->getServoMotor(1)->setTargetPosition(125, REPLACE);
		    }
                break;

    case 'o' : for(int i=0; i<atomNumber; i++) //Place toutes les pattes en position 0
                  for(int j=0; j<6; j++)
                    {
		    atome[i].getLeg(j)->getServoMotor(0)->setTargetPosition(0, REPLACE);
		    atome[i].getLeg(j)->getServoMotor(1)->setTargetPosition(0, REPLACE);
		    }
                break;

    case 'u' : for(int i=0; i<atomNumber; i++) //Place toutes les pattes en position 250
                  for(int j=0; j<6; j++)
                    {
		    atome[i].getLeg(j)->getServoMotor(0)->setTargetPosition(250, REPLACE);
		    atome[i].getLeg(j)->getServoMotor(1)->setTargetPosition(250, REPLACE);
		    }
                break;

    case 'g' : atome[atomeActif].getLeg(patteActive)->getServoMotor(0)->setTargetPosition(0, KEEP); break;
    case 'h' : atome[atomeActif].getLeg(patteActive)->getServoMotor(0)->setTargetPosition(125, KEEP); break;
    case 'j' : atome[atomeActif].getLeg(patteActive)->getServoMotor(0)->setTargetPosition(250, KEEP); break;


    case 'v' : atome[atomeActif].getLeg(patteActive)->getServoMotor(1)->setTargetPosition(0, KEEP); break;
    case 'b' : atome[atomeActif].getLeg(patteActive)->getServoMotor(1)->setTargetPosition(125, KEEP); break;
    case 'n' : atome[atomeActif].getLeg(patteActive)->getServoMotor(1)->setTargetPosition(250, KEEP); break;

    case 's' : sequence(); break;  //Excute la squence programme

    default : printf("Touche non reconnue : %c (%i)\n", cmd, cmd);
    }
  }

int main(int argc, char** argv)
  {
  CR_SETUP(Main, TARGET_FPS         , CR_DOUBLE);
  CR_SETUP(Main, GROUP_POSITION     , CR_DOUBLE);
  CR_SETUP(Main, ATOM_POSITION      , CR_DOUBLE);
  CR_SETUP(Main, GROUP_ATOM_COUNT   , CR_DOUBLE);
  CR_SETUP(Main, GROUP_ATOM_PER_ROW , CR_DOUBLE);
  CR_SETUP(Main, GROUP_ATOM_PER_LINE, CR_DOUBLE);

  CR_SETUP(Atom, KERNELSIDE, CR_DOUBLE);
  CR_SETUP(Atom, SPHERESIDE, CR_DOUBLE);
  CR_SETUP(Atom, LEGRADIUS , CR_DOUBLE);
  CR_SETUP(Atom, LEGSIDE   , CR_DOUBLE);

  char *odehome_env = getenv("ODEHOME");
  std::string odehome_string;

  #ifdef USE_OPENGL

  dsFunctions fn;

  fn.version = DS_VERSION;
  fn.start   = 0;
  fn.step    = &step;
  fn.command = &command;
  fn.stop    = &stop;

  if(odehome_env == NULL)
    odehome_string = "../../ode/drawstuff/textures";
  else
    {
    odehome_string = odehome_env;
    odehome_string += "/drawstuff/textures";
    }

  fn.path_to_textures = (char*)odehome_string.c_str();

  #endif

  engine = Engine::getInstance();
  engine->initialize();
  engine->setTargetFps(DVAR(TARGET_FPS));

  collisionGroup = new CollisionGroup();

  SimMolecule::setCollisionGroup(collisionGroup);

  CollisionManager::getInstance()->addCollisionGroup(collisionGroup);

  Node *root = engine->getRoot();

  collisionGroup->addCollidee(root);

  SimWorld *world = new SimWorld();

  root->addChild(world);

  atome = new API::Atom();
  atome->util->setCartesianPosition(new API::CartesianPosition(0, 0, 3));

  #ifdef USE_OPENGL

  printf("J'utilise OpenGL !\n");

  dsSimulationLoop(argc, argv, 640, 480, &fn);

  #else

  printf("Je n'utilise pas OpenGL !\n");

  bool quit  = false;
  bool pause = false;

  printf("Starting main loop...\n");

  engine->setIsPaused(true);

  while(! quit)
    step(pause);

  printf("Stopping !!!");

  stop();

  #endif

  return(0);
  }
