#include "pi2a14/pi2a14_araignee.h"
#include "pi2a14/pi2a14_utilitaires.h"

#include "ConfigReader.h"

/**
 * fonction globale qui permet de lier 2 atoms entre eux.
 * le principe de l'algorithme consiste a parcourire les 6 pattes du premier atome
 * pour chaque patte on verifie les 6 pattes de l'autre atome,
 * si il y a possibilite de lie les 2 pattes on les lies et on sort de la fonction.
 */
/*void linkAtom(API::Atom * atom1, API::Atom * atom2)
{
	// on initialise la variable qui verifie si il y a eut un lien entre les 2 atoms.
	bool link = false;
	for (int i = 0;i < 6; i++)
	{
		for (int j = 0; j < 6; j++)
		{
		    if (atom1->getLeg(i)->util->isConnectable(atom2->getLeg(j)))
		    {
			    atom1->getLeg(i)->setLink(atom2->getLeg(j));
			    printf("patte %d et patte %d connect\n",i,j);
			    link = true;
		    }
		    if (link)
		    {
			    break;
		    }
		}
	    if (link)
	    {
		    break;
	    }
	}

}*/

/**
 * constructeur par defaut de l'araigne.
 * ce constructeur cre 12 atoms et les lies entre eux 
 *       O--O--O--O                                       
 *          |  |     vue de haut                          
 *       O--O--O--O                                       
 *                                                        
 *       O--O--O--O                                       
 *       |        |                                       
 *       O        O  vue de cot                          
 *       |        |                           
 * une fois les 12 atomes cre et positionn dans l'espace, ils sont li entre eux, puis les 
 * structures de pattes (voir la classe Patte du projet (pas la classe Leg de l'API)) sont mises en places.
 */
Araignee::Araignee()
{
	// creation des 4 pattes
	pattes = new Patte[4];
	
	// les pattes 0 et 1 sont des pattes droite (par defaut une patte est gauche)
	// le fait de specifier que les pattes sont droites ou gauches permet d'obtenir un niveau
	// d'abstraction suffisant pour pouvoir par la suite envoyer des ordres au patte comme avancer ou strafer
	// sans avoir a s'en occuper (c'est la patte en interne qui choisit l'algorithme a utiliser.
	pattes[0].setIsGauche(false);
	pattes[1].setIsGauche(false);
	
	// declaration obligatoire pour connaitre les valeurs des dimension de l'atome (utilis pour positionner
	// les atomes dans l'espace.
	CR_DECLARE (KERNELSIDE);
	CR_DECLARE (SPHERESIDE);
	CR_DECLARE (LEGRADIUS);
	CR_DECLARE (LEGSIDE);
	
	CR_SETUP (Atom, KERNELSIDE, CR_DOUBLE);
	CR_SETUP (Atom, SPHERESIDE, CR_DOUBLE);
	CR_SETUP (Atom, LEGRADIUS, CR_DOUBLE);
	CR_SETUP (Atom, LEGSIDE, CR_DOUBLE);
	
	// creation des 12 atomes qui forment l'araigne
	atoms = new API::Atom[12];
	
	// on positionne les atomes pour cre la structure initiale.
	atoms[0].util->setCartesianPosition(new API::CartesianPosition(
									0,
									0,
									DVAR(KERNELSIDE)*1+2*1*DVAR(LEGSIDE)+0.75));
	atoms[1].util->setCartesianPosition(new API::CartesianPosition(
									-(DVAR(KERNELSIDE)*1+2*1*DVAR(LEGSIDE)),
									0,
									DVAR(KERNELSIDE)*1+2*1*DVAR(LEGSIDE)+0.75));
	atoms[2].util->setCartesianPosition(new API::CartesianPosition(
									-(DVAR(KERNELSIDE)*1+2*1*DVAR(LEGSIDE)),
									0,
									0.75));
	
	atoms[3].util->setCartesianPosition(new API::CartesianPosition(
									0,
									DVAR(KERNELSIDE)*1+2*1*DVAR(LEGSIDE),
									DVAR(KERNELSIDE)*1+2*1*DVAR(LEGSIDE)+0.75));
	atoms[4].util->setCartesianPosition(new API::CartesianPosition(
									-(DVAR(KERNELSIDE)*1+2*1*DVAR(LEGSIDE)),
									DVAR(KERNELSIDE)*1+2*1*DVAR(LEGSIDE),
									DVAR(KERNELSIDE)*1+2*1*DVAR(LEGSIDE)+0.75));
	atoms[5].util->setCartesianPosition(new API::CartesianPosition(
									-(DVAR(KERNELSIDE)*1+2*1*DVAR(LEGSIDE)),
									DVAR(KERNELSIDE)*1+2*1*DVAR(LEGSIDE),
									0.75));

	atoms[6].util->setCartesianPosition(new API::CartesianPosition(
									DVAR(KERNELSIDE)*1+2*1*DVAR(LEGSIDE),
									0,
									DVAR(KERNELSIDE)*1+2*1*DVAR(LEGSIDE)+0.75));
	atoms[7].util->setCartesianPosition(new API::CartesianPosition(
									DVAR(KERNELSIDE)*1+2*1*DVAR(LEGSIDE)+(DVAR(KERNELSIDE)*1+2*1*DVAR(LEGSIDE)),
									0,
									DVAR(KERNELSIDE)*1+2*1*DVAR(LEGSIDE)+0.75));
	atoms[8].util->setCartesianPosition(new API::CartesianPosition(
									DVAR(KERNELSIDE)*1+2*1*DVAR(LEGSIDE)+(DVAR(KERNELSIDE)*1+2*1*DVAR(LEGSIDE)),
									0,
									0.75));
	
	atoms[9].util->setCartesianPosition(new API::CartesianPosition(
									DVAR(KERNELSIDE)*1+2*1*DVAR(LEGSIDE),
									DVAR(KERNELSIDE)*1+2*1*DVAR(LEGSIDE),
									DVAR(KERNELSIDE)*1+2*1*DVAR(LEGSIDE)+0.75));
	atoms[10].util->setCartesianPosition(new API::CartesianPosition(
									DVAR(KERNELSIDE)*1+2*1*DVAR(LEGSIDE)+(DVAR(KERNELSIDE)*1+2*1*DVAR(LEGSIDE)),
									DVAR(KERNELSIDE)*1+2*1*DVAR(LEGSIDE),
									DVAR(KERNELSIDE)*1+2*1*DVAR(LEGSIDE)+0.75));
	atoms[11].util->setCartesianPosition(new API::CartesianPosition(
									DVAR(KERNELSIDE)*1+2*1*DVAR(LEGSIDE)+(DVAR(KERNELSIDE)*1+2*1*DVAR(LEGSIDE)),
									DVAR(KERNELSIDE)*1+2*1*DVAR(LEGSIDE),
									0.75));
	// phase de linkage des atomes. on essaye de connecter que les atomes qui on besoin d'etres connect entre eux
	linkAtom(&atoms[0],&atoms[1]);
	linkAtom(&atoms[1],&atoms[2]);
	linkAtom(&atoms[3],&atoms[4]);
	linkAtom(&atoms[4],&atoms[5]);
	linkAtom(&atoms[6],&atoms[7]);
	linkAtom(&atoms[7],&atoms[8]);
	linkAtom(&atoms[9],&atoms[10]);
	linkAtom(&atoms[10],&atoms[11]);
	linkAtom(&atoms[0],&atoms[3]);
	linkAtom(&atoms[0],&atoms[6]);
	linkAtom(&atoms[6],&atoms[9]);
	linkAtom(&atoms[9],&atoms[3]);

	// initialisation des divers variables
	
	// on dit au pattes si elle sont paire ou impaire, c'est utilis par le rammasage par exemple.
	pattes[0].setIsPaire(true);
	pattes[1].setIsPaire(false);
	pattes[2].setIsPaire(true);
	pattes[3].setIsPaire(false);
	
	// on donne 4 index de synchronisation, 1 a chaque patte, ainsi on peut savoir si la patte est en mouvement ou pas.
	pattes[0].setSyncIndex(getNextSyncIndex());
	pattes[1].setSyncIndex(getNextSyncIndex());
	pattes[2].setSyncIndex(getNextSyncIndex());
	pattes[3].setSyncIndex(getNextSyncIndex());
	
	// on ajoute les atomes aux pattes.
	pattes[0].ajoutAtom(&atoms[0],0);
	pattes[0].ajoutAtom(&atoms[1],1);
	pattes[0].ajoutAtom(&atoms[2],2);
	pattes[1].ajoutAtom(&atoms[3],0);
	pattes[1].ajoutAtom(&atoms[4],1);
	pattes[1].ajoutAtom(&atoms[5],2);
	pattes[2].ajoutAtom(&atoms[6],0);
	pattes[2].ajoutAtom(&atoms[7],1);
	pattes[2].ajoutAtom(&atoms[8],2);
	pattes[3].ajoutAtom(&atoms[9],0);
	pattes[3].ajoutAtom(&atoms[10],1);
	pattes[3].ajoutAtom(&atoms[11],2);
	
	// puis on les ajoute  la molecule.
	for (int i = 0; i < 12 ; i++)
	{
		this->addAtom(&atoms[i]);
	}
	
	// on donne le mouvement par defaut et l'etape par defaut.
	moveType = 0;
	etape = 0;

}

/**
 * initialise l'araigne dans sa position d'"equilibre", c'est a dire la position qu'elle a au repos.
 */
void Araignee::initialise()
{
	// on initialise chaque pattes
	for (int i = 0 ;i < 4; i++)
	{
		pattes[i].initialise();
	}
	// on met le mouvement par defaut c'est a dire pas de mouvement.
	moveType = 0;
	etape = 0;
}

/**
 * methode qui lance le mouvement de marche vers l'avant
 */
void Araignee::avance()
{
	printf("marche => etape 1\n");
	pattes[0].monte();
	pattes[3].avance();
	moveType = 1;
	etape = 1;
}


/**
 * destructeur par defaut
 */
Araignee::~Araignee()
{
	delete []pattes;
	// il faut peut etre delet les atomes, je ne sait pas.
}

/**
 * methode qui verifie les synchronisation des mouvements.
 * le principe de cette methode est que selon le type de mouvement,
 * selon l'etape courante, on verifie si le mouvement precedent est fini
 * et si il est fini on lance les ordres pour les mouvements suivante puis on passe  l'etape suivante.
 */
void Araignee::checkSynchro()
{
		// selon le type de mouvement
		switch(moveType)
		{
			case 1:
				// pour la marche les synchro sont toute base sur le mme principe donc on gagne du temps.
				if (!pattes[0].isMoving() && !pattes[1].isMoving() && !pattes[2].isMoving() && !pattes[3].isMoving()) // tout les mouvement sont termin
				{
					if (moveType != 0)
					{
						// on a fini l'etape precedente donc on passe a l'etape suivante
						// sauf pour le cas de l'initialisation...
						etape ++;
					}
				// selon le numero d'etape
				switch(etape)
				{
					case 1:
						// on monte la patte 0 et on avance la patte 3
						// le fait de faire monter la patte 0 bascule la molecule et donc on peut
						// avancer la patte 3
						printf("marche => etape 1\n");
						pattes[0].monte();
						pattes[3].avance();
						break;
					case 2:
						printf("marche => etape 2\n");
						// on reinitialise la patte pour la ramener au niveau du sol
						pattes[0].initialise();
						break;
					case 3:
						printf("marche => etape 3\n");
						// on fait comme pour l'etape 1 mais avec les autres pattes
						pattes[2].monte();
						pattes[1].avance();
						break;
					case 4:
						printf("marche => etape 4\n");
						// on reinitialise la patte pour la ramener au niveau du sol
						pattes[2].initialise();
						break;
					case 5:
						printf("marche => etape 5\n");
						// on deplace le corps de l'araigne de l'arriere vers l'avant.
						pattes[1].initialise();
						pattes[3].initialise();
						pattes[0].recule();
						pattes[2].recule();
						break;
					case 6:
						printf("marche => etape 6\n");
						pattes[3].monte();
						pattes[0].initialise();
						break;
					case 7:
						printf("marche => etape 7\n");
						// on reinitialise la patte pour la ramener au niveau du sol
						pattes[3].initialise();
						break;
					case 8:
						printf("marche => etape 8\n");
						pattes[1].monte();
						pattes[2].initialise();
						break;
					case 9:
						printf("marche => etape 9\n");
						// on reinitialise la patte pour la ramener au niveau du sol
						// full rinitialisation pour corriger les possibles defaut de replacements
						// due au force exerce sur les pattes
						pattes[0].initialise();
						pattes[1].initialise();
						pattes[2].initialise();
						pattes[3].initialise();
						break;
					case 10:
						avance();
						break;
				}
				break;
			case 2:
				switch (etape)
				{
					case 1:
						if ((!pattes[0].isMoving() && !pattes[1].isMoving() && !pattes[2].isMoving() && !pattes[3].isMoving())
							||(pattes[0].touchGround(0) || pattes[1].touchGround(0) || pattes[2].touchGround(0) || pattes[3].touchGround(0))) // tout les mouvement sont termin
						{
							printf("les pattes touchent le sol\n");
							printf("passage a l etape 2\n");
							// si les pattes touchent le sol on arrete tout mouvement.
							this->stop();
							attraper();
							etape =2;
						}
						break;
					case 2:
						if (!pattes[0].isMoving() && !pattes[1].isMoving() && !pattes[2].isMoving() && !pattes[3].isMoving()) // tout les mouvement sont termin
						{
							printf("passage a l etape 3\n");
							for (int i =0; i < 4 ; i++)
							{
								pattes[i].initialise();
							}
							etape =3;
						}
						break;
					case 3:
						if (!pattes[0].isMoving() && !pattes[1].isMoving() && !pattes[2].isMoving() && !pattes[3].isMoving()) // tout les mouvement sont termin
						{
							printf("arret\n");
							etape = 1;
							moveType = 0;
						}
						break;
				}
				break;
			case 3:
				switch (etape)
				{
					case 1:
						if (!pattes[1].isMoving() && !pattes[2].isMoving()) // tout les mouvement sont termin
						{
							printf("passage a l'etape 2\n");
							etape++;
							pattes[1].initialise();
							pattes[2].descendreSimple();
						}
						break;
					case 2:
						if (!pattes[1].isMoving() && !pattes[2].isMoving()) // tout les mouvement sont termin
						{
							printf("passage a l'etape 3\n");
							etape++;
							pattes[0].allonger();
							pattes[3].allonger();
						}
						break;
					case 3:
						if (!pattes[0].isMoving() && !pattes[3].isMoving()) // tout les mouvement sont termin
						{
							printf("passage a l'etape 4\n");
							etape++;
							pattes[0].initialise();
							pattes[3].descendreSimple();
						}
						break;
					case 4:
						if (!pattes[0].isMoving() && !pattes[3].isMoving()) // tout les mouvement sont termin
						{
							printf("passage a l'etape 5\n");
							etape++;
							pattes[2].initialise();
							pattes[3].initialise();
							pattes[0].allonger();
							pattes[1].allonger();
						}
						break;
					case 5:
						if (!pattes[0].isMoving() && !pattes[1].isMoving() && !pattes[2].isMoving() && !pattes[3].isMoving()) // tout les mouvement sont termin
						{
							printf("passage a l'etape 6\n");
							etape++;
							pattes[0].monte();
							pattes[3].monte();
						}
						break;
					case 6:
						if (!pattes[0].isMoving() && !pattes[3].isMoving()) // tout les mouvement sont termin
						{
							printf("passage a l'etape 7\n");
							etape++;
							pattes[0].initialise();
							pattes[3].initialise();
						}
						break;
					case 7:
						if (!pattes[0].isMoving() && !pattes[3].isMoving()) // tout les mouvement sont termin
						{
							printf("passage a l'etape 8\n");
							etape++;
							pattes[1].monte();
							pattes[2].monte();
						}
						break;
					case 8:
						if (!pattes[1].isMoving() && !pattes[2].isMoving()) // tout les mouvement sont termin
						{
							printf("passage a l'etape 9\n");
							etape++;
							pattes[0].initialise();
							pattes[3].initialise();
						}
						break;
					case 9:
						if (!pattes[1].isMoving() && !pattes[2].isMoving()) // tout les mouvement sont termin
						{
							printf("passage a l'etape 1\n");
							etape = 1;
						}
						break;
				}
		}
	}
}

void Araignee::ramasser()
{
	printf("etape 1\n");
	for (int i =0; i < 4 ; i++)
	{
		pattes[i].baisser();
	}
	moveType = 2;
	etape = 1;
}

void Araignee::attraper()
{
	printf("etape 1\n");
	for (int i =0; i < 4 ; i++)
	{
		pattes[i].attrape();
		printf("attrape\n");
	}

}

void Araignee::strafGauche()
{
	moveType = 3;
	etape = 1;
	pattes[2].allonger();
	pattes[1].monte();
}



