#include "pi2a14/pi2a14_patte.h"

Patte::Patte()
{
	isGauche = true;
	atom = new API::Atom*();
	syncIndex = 0; // mis a une valeur par defaut
}

Patte::~Patte()
{
}

void Patte::setIsGauche(bool gauche)
{
	isGauche = gauche;
}

void Patte::setIsPaire(bool paire)
{
	isPaire = paire;
}

/**
 * met l'atom pass en parametre dans le tableau d'atom a la place i
 * @param atom l'atom a mettre dans la patte
 * @param place la place de l'atom. la valeur doit etre comprise entre 0 et 2 inclu.
 */
void Patte::ajoutAtom(API::Atom * atom, int place)
{
	if (place >= 0 && place < 3)
	{
		this->atom[place] = atom;
	}
}


bool Patte::isMoving()
{
	return !atom[0]->getMolecule()->isFinished(syncIndex);
}

void Patte::initialise()
{
	if (atom[0]->getMolecule()->isFinished(syncIndex))
	{
		// initialisation particuliere pour le premier atom
		if (isGauche)
		{
			atom[0]->getLeg(0)->getServoMotor(0)->setTargetPosition(200,KEEP,0,syncIndex);
			atom[1]->getLeg(2)->getServoMotor(1)->setTargetPosition(50,KEEP,0,syncIndex);
			atom[0]->getLeg(0)->getServoMotor(1)->setTargetPosition(125,KEEP,0,syncIndex);
			atom[1]->getLeg(2)->getServoMotor(0)->setTargetPosition(125,KEEP,0,syncIndex);
			atom[1]->getLeg(1)->getServoMotor(0)->setTargetPosition(125,KEEP,0,syncIndex);
			atom[1]->getLeg(1)->getServoMotor(1)->setTargetPosition(125,KEEP,0,syncIndex);
			
		}
		else
		{
			atom[0]->getLeg(1)->getServoMotor(0)->setTargetPosition(50,KEEP,0,syncIndex);
			atom[1]->getLeg(2)->getServoMotor(1)->setTargetPosition(200,KEEP,0,syncIndex);
			atom[0]->getLeg(1)->getServoMotor(1)->setTargetPosition(125,KEEP,0,syncIndex);
			atom[1]->getLeg(2)->getServoMotor(0)->setTargetPosition(125,KEEP,0,syncIndex);
			atom[1]->getLeg(0)->getServoMotor(0)->setTargetPosition(125,KEEP,0,syncIndex);
			atom[1]->getLeg(0)->getServoMotor(1)->setTargetPosition(125,KEEP,0,syncIndex);
		}
	}

}

void Patte::monte()
{
	if (atom[0]->getMolecule()->isFinished(syncIndex))
	{
		if (isGauche)
		{
			atom[0]->getLeg(0)->getServoMotor(0)->setTargetPosition(240,KEEP,0,syncIndex);
			atom[1]->getLeg(2)->getServoMotor(1)->setTargetPosition(10,KEEP,0,syncIndex);
		}
		else
		{
			atom[0]->getLeg(1)->getServoMotor(0)->setTargetPosition(10,KEEP,0,syncIndex);
			atom[1]->getLeg(2)->getServoMotor(1)->setTargetPosition(240,KEEP,0,syncIndex);
		}
	}
}

void Patte::avance()
{
	if (atom[0]->getMolecule()->isFinished(syncIndex))
	{
		if (isGauche)
		{
			atom[0]->getLeg(0)->getServoMotor(1)->setTargetPosition(10,KEEP,0,syncIndex);
		}
		else
		{
			atom[0]->getLeg(1)->getServoMotor(1)->setTargetPosition(240,KEEP,0,syncIndex);
		}
	}
}

void Patte::recule()
{
	if (atom[0]->getMolecule()->isFinished(syncIndex))
	{
		if (isGauche)
		{
			atom[0]->getLeg(0)->getServoMotor(1)->setTargetPosition(240,KEEP,0,syncIndex);
		}
		else
		{
			atom[0]->getLeg(1)->getServoMotor(1)->setTargetPosition(10,KEEP,0,syncIndex);
		}
	}
}

void Patte::descend()
{
	if (atom[0]->getMolecule()->isFinished(syncIndex))
	{
		if (isGauche)
		{
			atom[0]->getLeg(0)->getServoMotor(0)->setTargetPosition(10,KEEP,0,syncIndex);
			atom[1]->getLeg(2)->getServoMotor(1)->setTargetPosition(240,KEEP,0,syncIndex);
		}
		else
		{
			atom[0]->getLeg(1)->getServoMotor(0)->setTargetPosition(240,KEEP,0,syncIndex);
			atom[1]->getLeg(2)->getServoMotor(1)->setTargetPosition(10,KEEP,0,syncIndex);
		}
	}
}

void Patte::setSyncIndex(unsigned int index)
{
	syncIndex = index;
}

unsigned int Patte::getSyncIndex()
{
	return syncIndex;
}

void Patte::baisser()
{
	if (atom[0]->getMolecule()->isFinished(syncIndex))
	{
		if (isGauche)
		{
			atom[0]->getLeg(0)->getServoMotor(0)->setTargetPosition(250,KEEP,0,syncIndex);
			atom[1]->getLeg(2)->getServoMotor(1)->setTargetPosition(125,KEEP,0,syncIndex);
		}
		else
		{
			atom[0]->getLeg(1)->getServoMotor(0)->setTargetPosition(0,KEEP,0,syncIndex);
			atom[1]->getLeg(2)->getServoMotor(1)->setTargetPosition(125,KEEP,0,syncIndex);
		}
	}

}
void Patte::attrape()
{
	if (isGauche)
	{
		if (isPaire)
		{
			atom[0]->getLeg(2)->getServoMotor(0)->setTargetPosition(10,REPLACE,0,syncIndex);
			atom[0]->getLeg(2)->getServoMotor(1)->setTargetPosition(10,REPLACE,0,syncIndex);
		}
		else
		{
			atom[0]->getLeg(2)->getServoMotor(0)->setTargetPosition(240,REPLACE,0,syncIndex);
			atom[0]->getLeg(2)->getServoMotor(1)->setTargetPosition(10,REPLACE,0,syncIndex);
		}
	}
	else
	{
		if (isPaire)
		{
			atom[0]->getLeg(2)->getServoMotor(0)->setTargetPosition(10,REPLACE,0,syncIndex);
			atom[0]->getLeg(2)->getServoMotor(1)->setTargetPosition(240,REPLACE,0,syncIndex);
		}
		else
		{
			atom[0]->getLeg(2)->getServoMotor(0)->setTargetPosition(240,REPLACE,0,syncIndex);
			atom[0]->getLeg(2)->getServoMotor(1)->setTargetPosition(240,REPLACE,0,syncIndex);
		}
	}

}

bool Patte::touchGround(int atom)
{
	if (atom < 0)
	{
		atom = 0;
	}
	if (atom > 2)
	{
		atom = 2;
	}
	bool ret = false;
	for (int i = 0; i < 6; i++)
	{
		ret = ret || this->atom[atom]->getLeg(i)->util->isOnGround();
	}
	if (ret)
	{
		printf("touche le sol\n");
	}
	return ret;
}

void Patte::allonger()
{
	if (atom[0]->getMolecule()->isFinished(syncIndex))
	{
		if (isGauche)
		{
			atom[0]->getLeg(0)->getServoMotor(0)->setTargetPosition(125,KEEP,0,syncIndex);
			atom[1]->getLeg(1)->getServoMotor(0)->setTargetPosition(0,KEEP,0,syncIndex);
			atom[1]->getLeg(2)->getServoMotor(1)->setTargetPosition(250,KEEP,0,syncIndex);
		}
		else
		{
			atom[0]->getLeg(1)->getServoMotor(0)->setTargetPosition(125,KEEP,0,syncIndex);
			atom[1]->getLeg(0)->getServoMotor(0)->setTargetPosition(250,KEEP,0,syncIndex);
			atom[1]->getLeg(2)->getServoMotor(1)->setTargetPosition(0,KEEP,0,syncIndex);
		}
	}

}

void Patte::descendreSimple()
{
	if (atom[0]->getMolecule()->isFinished(syncIndex))
	{
		if (isGauche)
		{
			atom[0]->getLeg(0)->getServoMotor(0)->setTargetPosition(40,KEEP,0,syncIndex);
		}
		else
		{
			atom[0]->getLeg(1)->getServoMotor(0)->setTargetPosition(210,KEEP,0,syncIndex);
		}
	}

}

void Patte::monteSimple()
{
	if (atom[0]->getMolecule()->isFinished(syncIndex))
	{
		if (isGauche)
		{
			atom[0]->getLeg(0)->getServoMotor(0)->setTargetPosition(210,KEEP,0,syncIndex);
		}
		else
		{
			atom[0]->getLeg(1)->getServoMotor(0)->setTargetPosition(40,KEEP,0,syncIndex);
		}
	}

}

