//------------------------------------------------------------------------------
#include "pi2a14/pi2a14_tapis.h"
#include "pi2a14/pi2a14_utilitaires.h"
//------------------------------------------------------------------------------
#include "ConfigReader.h"

#include <stdio.h>
//------------------------------------------------------------------------------
#define ATOM_UNIT (DVAR(KERNELSIDE)+2*DVAR(LEGSIDE))
//------------------------------------------------------------------------------
Tapis::Tapis(int aLongueur, int aLargeur)
  {
  CR_DECLARE (KERNELSIDE);
  CR_DECLARE (SPHERESIDE);
  CR_DECLARE (LEGRADIUS);
  CR_DECLARE (LEGSIDE);

  CR_SETUP(Atom, KERNELSIDE, CR_DOUBLE);
  CR_SETUP(Atom, SPHERESIDE, CR_DOUBLE);
  CR_SETUP(Atom, LEGRADIUS , CR_DOUBLE);
  CR_SETUP(Atom, LEGSIDE   , CR_DOUBLE);

  numAtomes = aLongueur * aLargeur;
  longueur = aLongueur;
  largeur = aLargeur;

  //Cration des atomes constituants le tapis
  atoms = new API::Atom[numAtomes];

  //Cration du tableau de mapping interne des atomes
  mapping = new int[numAtomes];

  //Rpartition dans l'espace des atomes
  for(int i=0; i<longueur; i++)
    for(int j=0; j<largeur; j++)
      atoms[(longueur * i) + j].util->setCartesianPosition(new API::CartesianPosition(ATOM_UNIT * i, ATOM_UNIT * j, 1));

  //Cration des liens (!!! A optimiser !!!)
  for(int i=0; i<numAtomes; i++)
    for(int j=0; j<numAtomes; j++)
      linkAtom(&atoms[i], &atoms[j]);
  
  //Direction et remapping
  setDirection(NORTH);
  }
//------------------------------------------------------------------------------
Tapis::~Tapis()
  {
  delete [] atoms;
  delete [] mapping;
  }
//------------------------------------------------------------------------------
void Tapis::init()
  {
  for(int i=0; i<numAtomes; i++) //Place toutes les pattes en position 0
    for(int j=0; j<6; j++)
      {
      atoms[i].getLeg(j)->getServoMotor(0)->setTargetPosition(125, REPLACE);
      atoms[i].getLeg(j)->getServoMotor(1)->setTargetPosition(125, REPLACE);
      }
  
  //Placement du nez
  metPatte(calcNez(), 3, AVANT);
  }
//------------------------------------------------------------------------------
void Tapis::setDirection(int aDirection)
  {
  //Redressement du nez;
  metPatte(calcNez(), 3, DROIT);

  switch(aDirection)
    {
    case NORTH :
      //Remapping des atomes pour la direction NORTH
     for(int i=0; i<numAtomes; i++)
        for(int j=0; j<largeur; j++)
          mapping[(i*largeur)+j]=i+j*largeur;

      //Enregistrement de la direction
      directionCourante = NORTH;
      break;

   case EAST :
     //Remapping des atomes pour la direction EAST
     for(int i=0; i<numAtomes; i++)
       mapping[i]=numAtomes-i-1;

     //Enregistrement de la direction
     directionCourante = EAST;
     break;

   case SOUTH :
     //Remapping des atomes pour la direction SOUTH
     for(int i=0; i<numAtomes; i++)
        for(int j=0; j<largeur; j++)
          mapping[(i*largeur)+j]=(numAtomes-i-1)-j*largeur;

     //Enregistrement de la direction
     directionCourante = SOUTH;
     break;

   case WEST :
     //Remapping des atomes pour la direction WEST
     for(int i=0; i<numAtomes; i++)
       mapping[i]=i;

     //Enregistrement de la direction
     directionCourante = WEST;
     break;
     }
  
  //Abaissement du nez
  metPatte(calcNez(), 3, AVANT);
  }
//------------------------------------------------------------------------------
int Tapis::getDirection()
  {
 return(directionCourante);
  }
//------------------------------------------------------------------------------
void Tapis::etape1()
  {
  for(int i = 0; i<longueur; i+=2) //Parcours les atomes pairs cot gauche
    metPatte(i, BLEUE, AVANT);
    
  for(int i=numAtomes-longueur+1; i<numAtomes; i+=2) //Parcours les atomes
    metPatte(i, BLEUE, AVANT);                       //impairs cot droit
  }
//------------------------------------------------------------------------------
void Tapis::etape2()
  {
  for(int i = 1; i<longueur; i+=2) //Parcours les atomes impairs cot gauche
    metPatte(i, BLEUE, ARRIERE);
    
  for(int i=numAtomes-longueur; i<numAtomes; i+=2) //Parcours les atomes
    metPatte(i, BLEUE, ARRIERE);                     //pair cot droit
  }
//------------------------------------------------------------------------------
void Tapis::etape3()
  {
  for(int i = 0; i<longueur; i+=2) //Parcours les atomes pairs cot gauche
    metPatte(i, BLEUE, DROIT);
    
  for(int i=numAtomes-longueur+1; i<numAtomes; i+=2) //Parcours les atomes
    metPatte(i, BLEUE, DROIT);                       //impairs cot droit
  }
//------------------------------------------------------------------------------
void Tapis::etape4()
  {
  for(int i = 1; i<longueur; i+=2) //Parcours les atomes impairs cot gauche
    metPatte(i, BLEUE, AVANT);
    
  for(int i=numAtomes-longueur; i<numAtomes; i+=2) //Parcours les atomes
    metPatte(i, BLEUE, AVANT);                     //pair cot droit
  }
//------------------------------------------------------------------------------
void Tapis::marche()
  {
  static int etape = 0;
  
  switch(etape)
    {
    case 0 : init(); 
             for(int i = longueur; i<numAtomes-longueur; i++)
	       metPatte(i, BLEUE, ARRIERE); 
             break;
    
    case 1 : etape1(); break;
    
    case 2 : etape2(); break;
    
    case 3 : etape3(); break;
    
    case 4 : etape4(); break;
    
    //case 5 : etape5(); break;
    }
  
  printf("etape = %i\n", etape);
    
  etape++;
    
  if(etape>4)  //Retour  l'tape 0
    etape=0;
  
  }
//------------------------------------------------------------------------------
void Tapis::metPatte(int aNumAtome, int aNumPatte, int aMvt)
  {
  if(aNumPatte != 2 && aNumPatte != 3) //Pattes du haut ou du bas
   return;

  if(aNumAtome > numAtomes-1 || aNumAtome < 0)  //Numros d'atomes qui n'existent pas
    return;

int motor=0;
int position=0;

  switch(aMvt)
    {
    case AVANT :

      if(directionCourante == NORTH)
        {
        if(aNumPatte == 2) {motor=1; position=0;}
        if(aNumPatte == 3) {motor=1; position=250;}  
        }

      if(directionCourante == SOUTH)
        {
        if(aNumPatte == 2) {motor=1; position=250;}
        if(aNumPatte == 3) {motor=1; position=0;}  
        }

      if(directionCourante == EAST)
        {
        if(aNumPatte == 2) {motor=0; position=0;}
        if(aNumPatte == 3) {motor=0; position=250;}  
        }

      if(directionCourante == WEST)
        {
        if(aNumPatte == 2) {motor=0; position=250;}
        if(aNumPatte == 3) {motor=0; position=0;}  
        }

      atoms[mapping[aNumAtome]].getLeg(aNumPatte)->getServoMotor(motor)->setTargetPosition(position, REPLACE);
      break;

    case ARRIERE :
      if(directionCourante == NORTH)
        {
        if(aNumPatte == 2) {motor=1; position=250;}
        if(aNumPatte == 3) {motor=1; position=0;}  
        }

      if(directionCourante == SOUTH)
        {
        if(aNumPatte == 2) {motor=1; position=0;}
        if(aNumPatte == 3) {motor=1; position=250;}  
        }

      if(directionCourante == EAST)
        {
        if(aNumPatte == 2) {motor=0; position=250;}
        if(aNumPatte == 3) {motor=0; position=0;}  
        }

      if(directionCourante == WEST)
        {
        if(aNumPatte == 2) {motor=0; position=0;}
        if(aNumPatte == 3) {motor=0; position=250;}  
        }

      atoms[mapping[aNumAtome]].getLeg(aNumPatte)->getServoMotor(motor)->setTargetPosition(position, REPLACE);
      break;

    case EXT :
      if(aNumAtome<longueur) //Atomes  gauche
        {
        if(directionCourante == NORTH)
          {
          if(aNumPatte == 2) {motor=0; position=250;}
          if(aNumPatte == 3) {motor=0; position=0;} 
          }

        if(directionCourante == SOUTH)
          {
          if(aNumPatte == 2) {motor=0; position=0;}
          if(aNumPatte == 3) {motor=0; position=250;}  
          }

        if(directionCourante == EAST)
          {
          if(aNumPatte == 2) {motor=1; position=250;}
          if(aNumPatte == 3) {motor=1; position=0;}  
          }

        if(directionCourante == WEST)
          {
          if(aNumPatte == 2) {motor=1; position=0;}
          if(aNumPatte == 3) {motor=1; position=250;}  
          }
	
        atoms[mapping[aNumAtome]].getLeg(aNumPatte)->getServoMotor(motor)->setTargetPosition(position, REPLACE);
        break;
        }

      if(aNumAtome>=(numAtomes-longueur)) //Atomes  droite
        {
        if(directionCourante == NORTH)
          {
          if(aNumPatte == 2) {motor=0; position=0;}
          if(aNumPatte == 3) {motor=0; position=250;} 
          }

        if(directionCourante == SOUTH)
          {
          if(aNumPatte == 2) {motor=0; position=250;}
          if(aNumPatte == 3) {motor=0; position=0;}  
          }

        if(directionCourante == EAST)
          {
          if(aNumPatte == 2) {motor=1; position=0;}
          if(aNumPatte == 3) {motor=1; position=250;}  
          }

        if(directionCourante == WEST)
          {
          if(aNumPatte == 2) {motor=1; position=250;}
          if(aNumPatte == 3) {motor=1; position=0;}  
          }
	
	atoms[mapping[aNumAtome]].getLeg(aNumPatte)->getServoMotor(motor)->setTargetPosition(position, REPLACE);
        break;
	}

      break;

    case INT :
      if(aNumAtome<longueur) //Atomes  gauche
        {
        if(directionCourante == NORTH)
          {
          if(aNumPatte == 2) {motor=0; position=0;}
          if(aNumPatte == 3) {motor=0; position=250;} 
          }

        if(directionCourante == SOUTH)
          {
          if(aNumPatte == 2) {motor=0; position=250;}
          if(aNumPatte == 3) {motor=0; position=0;}  
          }

        if(directionCourante == EAST)
          {
          if(aNumPatte == 2) {motor=1; position=0;}
          if(aNumPatte == 3) {motor=1; position=250;}  
          }

        if(directionCourante == WEST)
          {
          if(aNumPatte == 2) {motor=1; position=250;}
          if(aNumPatte == 3) {motor=1; position=0;}  
          }
	
        atoms[mapping[aNumAtome]].getLeg(aNumPatte)->getServoMotor(motor)->setTargetPosition(position, REPLACE);
        break;
        }

      if(aNumAtome>=(numAtomes-longueur)) //Atomes  droite
        {
        if(directionCourante == NORTH)
          {
          if(aNumPatte == 2) {motor=0; position=250;}
          if(aNumPatte == 3) {motor=0; position=0;} 
          }

        if(directionCourante == SOUTH)
          {
          if(aNumPatte == 2) {motor=0; position=0;}
          if(aNumPatte == 3) {motor=0; position=250;}  
          }

        if(directionCourante == EAST)
          {
          if(aNumPatte == 2) {motor=1; position=250;}
          if(aNumPatte == 3) {motor=1; position=0;}  
          }

        if(directionCourante == WEST)
          {
          if(aNumPatte == 2) {motor=1; position=0;}
          if(aNumPatte == 3) {motor=1; position=250;}  
          }
	
	atoms[mapping[aNumAtome]].getLeg(aNumPatte)->getServoMotor(motor)->setTargetPosition(position, REPLACE);
        break;
	}

      break;

      break;

    case DROIT :
      atoms[mapping[aNumAtome]].getLeg(aNumPatte)->getServoMotor(0)->setTargetPosition(125, REPLACE);
      atoms[mapping[aNumAtome]].getLeg(aNumPatte)->getServoMotor(1)->setTargetPosition(125, REPLACE);
      break;
    }
  }
//------------------------------------------------------------------------------
int Tapis::calcNez()
  {
  return(((int)(largeur/2))*longueur);
  }
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
