#include "pi2a14/pi2a14_walker.h"

#include "ConfigReader.h"
#include "Atom.h"

//-------------------------------------------------------
Walker::Walker()
  {
	  
  CR_DECLARE (KERNELSIDE);
  CR_DECLARE (SPHERESIDE);
  CR_DECLARE (LEGRADIUS);
  CR_DECLARE (LEGSIDE);
  
  CR_SETUP (Atom, KERNELSIDE, CR_DOUBLE);
  CR_SETUP (Atom, SPHERESIDE, CR_DOUBLE);
  CR_SETUP (Atom, LEGRADIUS, CR_DOUBLE);
  CR_SETUP (Atom, LEGSIDE, CR_DOUBLE);

 etape = 0;
 patteStraf = 0;
  
  plan = new API::Atom[6];
  
   for (int i = 0; i <  3 ; i++) //Cration des ranges d'atomes
   {
	    plan[i].util->setCartesianPosition(new API::CartesianPosition(
	                                                                  DVAR(KERNELSIDE)*i+2*i*DVAR(LEGSIDE),
	                                                                  0,
	                                                                  0.75));
   }
  
  for ( int i = 3; i <  6 ; i++)
  {
	    plan[i].util->setCartesianPosition(new API::CartesianPosition(
	                                                                  DVAR(KERNELSIDE)*(i-3)+2*(i-3)*DVAR(LEGSIDE),
	                                                                  DVAR(KERNELSIDE)+2*DVAR(LEGSIDE),
	                                                                  0.75));
  }
  //Connection des atomes
  for (int i = 0; i < 5; i++)
	  {
	    if (plan[i].getLeg(0)->util->isConnectable(plan[i+1].getLeg(1)))
	      plan[i].getLeg(0)->setLink(plan[i+1].getLeg(1));
	  }
	  
  plan[1].getLeg(4)->setLink(plan[4].getLeg(5));  //Connexion de la transversale
  for (int i = 0;i < 6 ; i++)
  {
	  this->addAtom(&plan[i]);
  }
  
  syncIndex = getNextSyncIndex();
  moveType = 1;//mouvement de base => la marche
  }
//-------------------------------------------------------
Walker::~Walker()
  {
  delete[] plan;
  
  }
//-------------------------------------------------------
void Walker::initStruct()
  {
  //Rinitialisation
  for(int i=0; i<6; i++)
    for(int j=0; j<6; j++)
      {
      plan[i].getLeg(j)->getServoMotor(0)->setTargetPosition(125,REPLACE);
      plan[i].getLeg(j)->getServoMotor(1)->setTargetPosition(125,REPLACE);
      }
  
  //Mise en place de la forme dfinitive
  
  printf ("\nInitialisation\n");
  plan[1].getLeg(0)->getServoMotor(0)->setTargetPosition(75,REPLACE,0,syncIndex);
  plan[4].getLeg(0)->getServoMotor(0)->setTargetPosition(75,REPLACE,0,syncIndex);
  
  plan[1].getLeg(1)->getServoMotor(0)->setTargetPosition(175,REPLACE,0,syncIndex);
  plan[4].getLeg(1)->getServoMotor(0)->setTargetPosition(175,REPLACE,0,syncIndex);
  
  plan[0].getLeg(0)->getServoMotor(0)->setTargetPosition(175,REPLACE,0,syncIndex);
  plan[3].getLeg(0)->getServoMotor(0)->setTargetPosition(175,REPLACE,0,syncIndex);
  
  plan[2].getLeg(1)->getServoMotor(0)->setTargetPosition(75,REPLACE,0,syncIndex);
  plan[5].getLeg(1)->getServoMotor(0)->setTargetPosition(75,REPLACE,0,syncIndex);
  etape = 0;
  }
//-------------------------------------------------------

void Walker::walk()
  {
	 
	printf("etape %d\n",etape);  
	if (etape ==0)
	{
		etape = 1;
		walkOne();
	}
  }
  

//-------------------------------------------------------
void Walker::walkOne()
{
	
	printf ("\nwalkOne()\n");
	etape = 1;
	this->getAtom(1)->getLeg(1)->getServoMotor(0)->setTargetPosition(140,KEEP,0,syncIndex);
	this->getAtom(4)->getLeg(0)->getServoMotor(1)->setTargetPosition(0,KEEP,0,syncIndex);
	this->getAtom(1)->getLeg(1)->getServoMotor(0)->setTargetPosition(175,KEEP,0,syncIndex);
	
}

void Walker::walkTwo()
{
	
	printf ("\nwalkTwo()\n");
	etape = 2;
	this->getAtom(1)->getLeg(0)->getServoMotor(0)->setTargetPosition(110,KEEP,0,syncIndex);
	this->getAtom(4)->getLeg(1)->getServoMotor(1)->setTargetPosition(250,KEEP,0,syncIndex);
	this->getAtom(1)->getLeg(0)->getServoMotor(0)->setTargetPosition(75,KEEP,0,syncIndex);
	
	
}
void Walker::walkThree()
{
	
	printf ("\nwalkThree()\n");
	etape = 3;
	this->getAtom(4)->getLeg(0)->getServoMotor(1)->setTargetPosition(125,KEEP,0,syncIndex);
	this->getAtom(4)->getLeg(1)->getServoMotor(1)->setTargetPosition(125,KEEP,0,syncIndex);
	this->getAtom(1)->getLeg(0)->getServoMotor(1)->setTargetPosition(250,KEEP,0,syncIndex);
	this->getAtom(1)->getLeg(1)->getServoMotor(1)->setTargetPosition(0,KEEP,0,syncIndex);
	
	
}
void Walker::walkFour()
{
	
	printf ("\nwalkFour()\n");
	etape = 4;
	this->getAtom(4)->getLeg(1)->getServoMotor(0)->setTargetPosition(140,KEEP,0,syncIndex);
	this->getAtom(1)->getLeg(0)->getServoMotor(1)->setTargetPosition(125,KEEP,0,syncIndex);
	this->getAtom(4)->getLeg(1)->getServoMotor(0)->setTargetPosition(175,KEEP,0,syncIndex);
	
}
void Walker::walkFive()
{
	
	printf ("\nwalkFive()\n");
	etape = 5;
	this->getAtom(4)->getLeg(0)->getServoMotor(0)->setTargetPosition(110,KEEP,0,syncIndex);
	this->getAtom(1)->getLeg(1)->getServoMotor(1)->setTargetPosition(125,KEEP,0,syncIndex);
	this->getAtom(4)->getLeg(0)->getServoMotor(0)->setTargetPosition(75,KEEP,0,syncIndex);
	
}

//-------------------------------------------------------
bool Walker::patteIsMoving(API::Atom * atome, int patteNumber)
{
	int marge = 112;
	return 	(((atome->getLeg(patteNumber)->getServoMotor(0)->getCurrentPosition() > atome->getLeg(patteNumber)->getServoMotor(0)->getTargetPosition()+marge)||
		(atome->getLeg(patteNumber)->getServoMotor(0)->getCurrentPosition() < atome->getLeg(patteNumber)->getServoMotor(0)->getTargetPosition()-marge))||
		((atome->getLeg(patteNumber)->getServoMotor(1)->getCurrentPosition() > atome->getLeg(patteNumber)->getServoMotor(1)->getTargetPosition()+marge)||
		(atome->getLeg(patteNumber)->getServoMotor(1)->getCurrentPosition() < atome->getLeg(patteNumber)->getServoMotor(1)->getTargetPosition()-marge)));
}

bool Walker::isMoving(API::Atom * atome)
{
	bool ret = false;
	int i = 0;
	ret = (ret || patteIsMoving(&atome[i], 0));
	ret = (ret || patteIsMoving(&atome[i], 1));
	return ret;
}

bool Walker::isStopped(API::Atom * atome, int nbrAtome)
{
	bool ret = true;
	int i = 0;
	while ((ret)&&(i<nbrAtome))
	{
		ret = !isMoving(&atome[i]);
		i++;
	}
	return ret;
}

//-------------------------------------------------------
void Walker::checkSynchro()
{

	if (isFinished(syncIndex)){
		if (etape != 0)
		{
		etape++;
		}
		switch (moveType)
		{
			case 1:// la marche
				switch (etape)
				{
					case 1:
						walkOne();
						break;
					case 2:
						walkTwo();
						break;
					case 3:
						walkThree();
						break;
					case 4:
						walkFour();
						break;
					case 5:
						walkFive();
						break;
					case 6:
						initStruct();
						walkOne();
				}
				break;
		case 2: //Tourne a droite
				switch (etape)
				{
					case 1:
						goRightOne();
						break;
					case 2:
						goRightTwo();
						break;
					case 3:
						goRightThree();
						break;
					case 4:
						goRightFour();
						break;
					case 5:
						initStruct();
						break;
				}
				break;
		case 3: //Tourne  gauche
			switch (etape)
				{
					case 1:
						goLeftOne();
						break;
					case 2:
						goLeftTwo();
						break;
					case 3:
						goLeftThree();
						break;
					case 4:
						goLeftFour();
						break;
					case 5:
						initStruct();
						break;
				}
				break;
		case 4: //Straf  droite
				switch (etape)
				{
					case 1:
						straf1();
						break;
					case 2:
						straf2();
						break;
					case 3:
						straf3();
						break;
				}
				break;
		case 5: //Autre manire d'avancer
				switch (etape)
				{
					case 1:
						triang1();
						break;
					case 2:
						triang2();
						break;
					case 3:
						triang3();
						triang2();
						break;
				}
				break;
		case 6: //Straf  gauche
				switch (etape)
				{
					case 1:
						straf1();
						break;
					case 2:
						straf2();
						break;
					case 3:
						straf3();
						break;
				}
				break;
		}
	}
}
//-------------------------------------------------------
void Walker::goRightOne()
{
	printf ("\ngoRightOne\n");
	this->getAtom(1)->getLeg(1)->getServoMotor(0)->setTargetPosition(125,KEEP,0,syncIndex);
	this->getAtom(1)->getLeg(0)->getServoMotor(0)->setTargetPosition(125,KEEP,0,syncIndex);
	this->getAtom(4)->getLeg(1)->getServoMotor(0)->setTargetPosition(125,KEEP,0,syncIndex);
	this->getAtom(4)->getLeg(0)->getServoMotor(0)->setTargetPosition(125,KEEP,0,syncIndex);
	
	this->getAtom(2)->getLeg(1)->getServoMotor(0)->setTargetPosition(125,KEEP,0,syncIndex);
	this->getAtom(3)->getLeg(0)->getServoMotor(0)->setTargetPosition(125,KEEP,0,syncIndex);
	this->getAtom(0)->getLeg(0)->getServoMotor(0)->setTargetPosition(125,KEEP,0,syncIndex);
	this->getAtom(5)->getLeg(1)->getServoMotor(0)->setTargetPosition(125,KEEP,0,syncIndex);
	
      
	etape = 1;
}
//-------------------------------------------------------
void Walker::goRightTwo()
{
	printf ("\ngoRightTwo\n");
	
	this->getAtom(1)->getLeg(1)->getServoMotor(0)->setTargetPosition(100,KEEP,0,syncIndex);
	this->getAtom(4)->getLeg(0)->getServoMotor(0)->setTargetPosition(150,KEEP,0,syncIndex);
	
	this->getAtom(1)->getLeg(1)->getServoMotor(1)->setTargetPosition(70,KEEP,0,syncIndex);
	this->getAtom(4)->getLeg(0)->getServoMotor(1)->setTargetPosition(70,KEEP,0,syncIndex);
	
	this->getAtom(1)->getLeg(1)->getServoMotor(0)->setTargetPosition(125,KEEP,0,syncIndex);
	this->getAtom(4)->getLeg(0)->getServoMotor(0)->setTargetPosition(125,KEEP,0,syncIndex);
	
	
	this->getAtom(1)->getLeg(0)->getServoMotor(0)->setTargetPosition(100,KEEP,0,syncIndex);
	this->getAtom(4)->getLeg(1)->getServoMotor(0)->setTargetPosition(150,KEEP,0,syncIndex);
	
	this->getAtom(1)->getLeg(0)->getServoMotor(1)->setTargetPosition(70,KEEP,0,syncIndex);
	this->getAtom(4)->getLeg(1)->getServoMotor(1)->setTargetPosition(70,KEEP,0,syncIndex);
	
	this->getAtom(1)->getLeg(0)->getServoMotor(0)->setTargetPosition(125,KEEP,0,syncIndex);
	this->getAtom(4)->getLeg(1)->getServoMotor(0)->setTargetPosition(125,KEEP,0,syncIndex);
	
	etape = 2;
}
//-------------------------------------------------------
void Walker::goRightThree()
{
	printf("\ngoRightThree\n");
	
	this->getAtom(4)->getLeg(1)->getServoMotor(0)->setTargetPosition(100,KEEP,0,syncIndex);
	this->getAtom(1)->getLeg(0)->getServoMotor(0)->setTargetPosition(150,KEEP,0,syncIndex);
	
	this->getAtom(4)->getLeg(1)->getServoMotor(1)->setTargetPosition(125,KEEP,0,syncIndex);
	this->getAtom(1)->getLeg(0)->getServoMotor(1)->setTargetPosition(125,KEEP,0,syncIndex);
	
	this->getAtom(4)->getLeg(1)->getServoMotor(0)->setTargetPosition(125,KEEP,0,syncIndex);
	this->getAtom(1)->getLeg(0)->getServoMotor(0)->setTargetPosition(125,KEEP,0,syncIndex);
	
	etape = 3;
}
//-------------------------------------------------------
void Walker::goRightFour()
{
	printf("\ngoRightFour\n");
	
	this->getAtom(4)->getLeg(0)->getServoMotor(0)->setTargetPosition(150,KEEP,0,syncIndex);
	this->getAtom(1)->getLeg(1)->getServoMotor(0)->setTargetPosition(100,KEEP,0,syncIndex);
	
	this->getAtom(4)->getLeg(0)->getServoMotor(1)->setTargetPosition(125,KEEP,0,syncIndex);
	this->getAtom(1)->getLeg(1)->getServoMotor(1)->setTargetPosition(125,KEEP,0,syncIndex);
	
	this->getAtom(4)->getLeg(0)->getServoMotor(0)->setTargetPosition(125,KEEP,0,syncIndex);
	this->getAtom(1)->getLeg(1)->getServoMotor(0)->setTargetPosition(125,KEEP,0,syncIndex);
}
//-------------------------------------------------------
void Walker::turnRight()
{
	moveType = 2;
	goRightOne();
}
//-------------------------------------------------------
void Walker::goLeftOne()
{
	printf ("\ngoLeftOne\n");
	
	this->getAtom(1)->getLeg(0)->getServoMotor(0)->setTargetPosition(125,KEEP,0,syncIndex);
	this->getAtom(1)->getLeg(1)->getServoMotor(0)->setTargetPosition(125,KEEP,0,syncIndex);
	this->getAtom(4)->getLeg(0)->getServoMotor(0)->setTargetPosition(125,KEEP,0,syncIndex);
	this->getAtom(4)->getLeg(1)->getServoMotor(0)->setTargetPosition(125,KEEP,0,syncIndex);
	
	this->getAtom(2)->getLeg(1)->getServoMotor(0)->setTargetPosition(125,KEEP,0,syncIndex);
	this->getAtom(3)->getLeg(0)->getServoMotor(0)->setTargetPosition(125,KEEP,0,syncIndex);
	this->getAtom(0)->getLeg(0)->getServoMotor(0)->setTargetPosition(125,KEEP,0,syncIndex);
	this->getAtom(5)->getLeg(1)->getServoMotor(0)->setTargetPosition(125,KEEP,0,syncIndex);
	
      
	etape = 1;
}
//-------------------------------------------------------
void Walker::goLeftTwo()
{
	printf ("\ngoLeftTwo\n");
	
	this->getAtom(1)->getLeg(0)->getServoMotor(0)->setTargetPosition(100,KEEP,0,syncIndex);
	this->getAtom(4)->getLeg(1)->getServoMotor(0)->setTargetPosition(150,KEEP,0,syncIndex);
	
	this->getAtom(1)->getLeg(0)->getServoMotor(1)->setTargetPosition(180,KEEP,0,syncIndex);
	this->getAtom(4)->getLeg(1)->getServoMotor(1)->setTargetPosition(180,KEEP,0,syncIndex);
	
	this->getAtom(1)->getLeg(0)->getServoMotor(0)->setTargetPosition(125,KEEP,0,syncIndex);
	this->getAtom(4)->getLeg(1)->getServoMotor(0)->setTargetPosition(125,KEEP,0,syncIndex);
	
	
	this->getAtom(1)->getLeg(1)->getServoMotor(0)->setTargetPosition(100,KEEP,0,syncIndex);
	this->getAtom(4)->getLeg(0)->getServoMotor(0)->setTargetPosition(150,KEEP,0,syncIndex);
	
	this->getAtom(1)->getLeg(1)->getServoMotor(1)->setTargetPosition(180,KEEP,0,syncIndex);
	this->getAtom(4)->getLeg(0)->getServoMotor(1)->setTargetPosition(180,KEEP,0,syncIndex);
	
	this->getAtom(1)->getLeg(1)->getServoMotor(0)->setTargetPosition(125,KEEP,0,syncIndex);
	this->getAtom(4)->getLeg(0)->getServoMotor(0)->setTargetPosition(125,KEEP,0,syncIndex);
	
	etape = 2;
}
//-------------------------------------------------------
void Walker::goLeftThree()
{
	printf("\ngoLeftThree\n");
	
	this->getAtom(4)->getLeg(0)->getServoMotor(0)->setTargetPosition(150,KEEP,0,syncIndex);
	this->getAtom(1)->getLeg(1)->getServoMotor(0)->setTargetPosition(100,KEEP,0,syncIndex);
	
	this->getAtom(4)->getLeg(0)->getServoMotor(1)->setTargetPosition(125,KEEP,0,syncIndex);
	this->getAtom(1)->getLeg(1)->getServoMotor(1)->setTargetPosition(125,KEEP,0,syncIndex);
	
	this->getAtom(4)->getLeg(0)->getServoMotor(0)->setTargetPosition(125,KEEP,0,syncIndex);
	this->getAtom(1)->getLeg(1)->getServoMotor(0)->setTargetPosition(125,KEEP,0,syncIndex);
	
	etape = 3;
}
//-------------------------------------------------------
void Walker::goLeftFour()
{
	printf("\ngoLeftFour\n");
	
	this->getAtom(4)->getLeg(1)->getServoMotor(0)->setTargetPosition(100,KEEP,0,syncIndex);
	this->getAtom(1)->getLeg(0)->getServoMotor(0)->setTargetPosition(150,KEEP,0,syncIndex);
	
	this->getAtom(4)->getLeg(1)->getServoMotor(1)->setTargetPosition(125,KEEP,0,syncIndex);
	this->getAtom(1)->getLeg(0)->getServoMotor(1)->setTargetPosition(125,KEEP,0,syncIndex);
	
	this->getAtom(4)->getLeg(1)->getServoMotor(0)->setTargetPosition(125,KEEP,0,syncIndex);
	this->getAtom(1)->getLeg(0)->getServoMotor(0)->setTargetPosition(125,KEEP,0,syncIndex);
}
//-------------------------------------------------------
void Walker::turnLeft()
{
	moveType = 3;
	goLeftOne();
}
//-------------------------------------------------------
void Walker::straf1()
{
	printf("\nstraf1()\n");
	this->getAtom(0)->getLeg(2)->getServoMotor(1)->setTargetPosition(patteStraf,KEEP,0,syncIndex);
	this->getAtom(2)->getLeg(2)->getServoMotor(1)->setTargetPosition(patteStraf,KEEP,0,syncIndex);
	this->getAtom(3)->getLeg(2)->getServoMotor(1)->setTargetPosition(patteStraf,KEEP,0,syncIndex);
	this->getAtom(5)->getLeg(2)->getServoMotor(1)->setTargetPosition(patteStraf,KEEP,0,syncIndex);
	etape =1;
	
}
//-------------------------------------------------------
void Walker::straf2()
{
	printf("\nstraf2()\n");
	this->getAtom(0)->getLeg(0)->getServoMotor(0)->setTargetPosition(240,KEEP,0,syncIndex);
	this->getAtom(3)->getLeg(0)->getServoMotor(0)->setTargetPosition(240,KEEP,0,syncIndex);
	
	this->getAtom(1)->getLeg(1)->getServoMotor(0)->setTargetPosition(230,KEEP,0,syncIndex);
	this->getAtom(4)->getLeg(1)->getServoMotor(0)->setTargetPosition(230,KEEP,0,syncIndex);
	this->getAtom(1)->getLeg(0)->getServoMotor(0)->setTargetPosition(20,KEEP,0,syncIndex);
	this->getAtom(4)->getLeg(0)->getServoMotor(0)->setTargetPosition(20,KEEP,0,syncIndex);
	
	this->getAtom(2)->getLeg(1)->getServoMotor(0)->setTargetPosition(20,KEEP,0,syncIndex);
	this->getAtom(5)->getLeg(1)->getServoMotor(0)->setTargetPosition(20,KEEP,0,syncIndex);
	etape =2;
	
	
}
//-------------------------------------------------------
void Walker::straf3()
{
	printf("\nstraf3()\n");
	plan[1].getLeg(0)->getServoMotor(0)->setTargetPosition(75,REPLACE,0,syncIndex);
	plan[4].getLeg(0)->getServoMotor(0)->setTargetPosition(75,REPLACE,0,syncIndex);
  
  	plan[1].getLeg(1)->getServoMotor(0)->setTargetPosition(175,REPLACE,0,syncIndex);
	plan[4].getLeg(1)->getServoMotor(0)->setTargetPosition(175,REPLACE,0,syncIndex);
  
  	plan[0].getLeg(0)->getServoMotor(0)->setTargetPosition(175,REPLACE,0,syncIndex);
	plan[3].getLeg(0)->getServoMotor(0)->setTargetPosition(175,REPLACE,0,syncIndex);
  
  	plan[2].getLeg(1)->getServoMotor(0)->setTargetPosition(75,REPLACE,0,syncIndex);
	plan[5].getLeg(1)->getServoMotor(0)->setTargetPosition(75,REPLACE,0,syncIndex);
	etape =3;
	
	
}
//-------------------------------------------------------
void Walker::strafRight() // lance le straf droite
{
	patteStraf = 0;
	straf1();
	moveType = 4;
	
}
//-------------------------------------------------------
void Walker::strafLeft() // lance le straf left
{
	patteStraf = 240;
	straf1();
	moveType = 6;
	
}
//-------------------------------------------------------
void Walker::triang1()
{
	printf("\ntriang1\n");
	
	
	this->getAtom(1)->getLeg(0)->getServoMotor(0)->setTargetPosition(125,KEEP,0,syncIndex);
	this->getAtom(1)->getLeg(1)->getServoMotor(0)->setTargetPosition(125,KEEP,0,syncIndex);
	this->getAtom(4)->getLeg(0)->getServoMotor(0)->setTargetPosition(125,KEEP,0,syncIndex);
	this->getAtom(4)->getLeg(1)->getServoMotor(0)->setTargetPosition(125,KEEP,0,syncIndex);
	
	this->getAtom(2)->getLeg(1)->getServoMotor(0)->setTargetPosition(125,KEEP,0,syncIndex);
	this->getAtom(3)->getLeg(0)->getServoMotor(0)->setTargetPosition(125,KEEP,0,syncIndex);
	this->getAtom(0)->getLeg(0)->getServoMotor(0)->setTargetPosition(125,KEEP,0,syncIndex);
	this->getAtom(5)->getLeg(1)->getServoMotor(0)->setTargetPosition(125,KEEP,0,syncIndex);
	
	this->getAtom(2)->getLeg(1)->getServoMotor(0)->setTargetPosition(240,KEEP,0,syncIndex);
	this->getAtom(3)->getLeg(0)->getServoMotor(0)->setTargetPosition(10,KEEP,0,syncIndex);
	this->getAtom(0)->getLeg(0)->getServoMotor(0)->setTargetPosition(10,KEEP,0,syncIndex);
	this->getAtom(5)->getLeg(1)->getServoMotor(0)->setTargetPosition(240,KEEP,0,syncIndex);
	
	etape = 1;
}
//-------------------------------------------------------
void Walker::triang2()
{
	printf("\ntriang2\n");
	
	this->getAtom(1)->getLeg(0)->getServoMotor(0)->setTargetPosition(150,KEEP,0,syncIndex);
	this->getAtom(1)->getLeg(1)->getServoMotor(0)->setTargetPosition(100,KEEP,0,syncIndex);
	this->getAtom(4)->getLeg(0)->getServoMotor(0)->setTargetPosition(150,KEEP,0,syncIndex);
	this->getAtom(4)->getLeg(1)->getServoMotor(0)->setTargetPosition(100,KEEP,0,syncIndex);
	
	this->getAtom(4)->getLeg(0)->getServoMotor(1)->setTargetPosition(75,KEEP,0,syncIndex);
	this->getAtom(4)->getLeg(1)->getServoMotor(1)->setTargetPosition(175,KEEP,0,syncIndex);
	this->getAtom(1)->getLeg(0)->getServoMotor(1)->setTargetPosition(75,KEEP,0,syncIndex);
	this->getAtom(1)->getLeg(1)->getServoMotor(1)->setTargetPosition(175,KEEP,0,syncIndex);
	
	this->getAtom(4)->getLeg(0)->getServoMotor(0)->setTargetPosition(125,KEEP,0,syncIndex);
	this->getAtom(4)->getLeg(1)->getServoMotor(0)->setTargetPosition(125,KEEP,0,syncIndex);
	this->getAtom(1)->getLeg(0)->getServoMotor(0)->setTargetPosition(125,KEEP,0,syncIndex);
	this->getAtom(1)->getLeg(1)->getServoMotor(0)->setTargetPosition(125,KEEP,0,syncIndex);
	
	etape = 2;
}
//-------------------------------------------------------
void Walker::triang3()
{
	printf("\ntriang3\n");
	
	this->getAtom(1)->getLeg(0)->getServoMotor(0)->setTargetPosition(100,KEEP,0,syncIndex);
	this->getAtom(4)->getLeg(0)->getServoMotor(0)->setTargetPosition(100,KEEP,0,syncIndex);
	this->getAtom(1)->getLeg(1)->getServoMotor(0)->setTargetPosition(150,KEEP,0,syncIndex);
	this->getAtom(4)->getLeg(1)->getServoMotor(0)->setTargetPosition(150,KEEP,0,syncIndex);
	
	this->getAtom(1)->getLeg(0)->getServoMotor(1)->setTargetPosition(125,KEEP,0,syncIndex);
	this->getAtom(4)->getLeg(0)->getServoMotor(1)->setTargetPosition(125,KEEP,0,syncIndex);
	this->getAtom(1)->getLeg(1)->getServoMotor(1)->setTargetPosition(125,KEEP,0,syncIndex);
	this->getAtom(4)->getLeg(1)->getServoMotor(1)->setTargetPosition(125,KEEP,0,syncIndex);
	
	this->getAtom(1)->getLeg(0)->getServoMotor(0)->setTargetPosition(125,KEEP,0,syncIndex);
	this->getAtom(4)->getLeg(0)->getServoMotor(0)->setTargetPosition(125,KEEP,0,syncIndex);
	this->getAtom(1)->getLeg(0)->getServoMotor(0)->setTargetPosition(125,KEEP,0,syncIndex);
	this->getAtom(4)->getLeg(0)->getServoMotor(0)->setTargetPosition(125,KEEP,0,syncIndex);
	
	etape = 3;
}
//-------------------------------------------------------
void Walker::triang()
{
	triang1();
	moveType = 5;
}

//-------------------------------------------------------
//-------------------------------------------------------
//-------------------------------------------------------
//-------------------------------------------------------
